class TrackPt {
  int pt_id;
  float lon;
  float lat;
  float speed;
  float heading;
  String name;
  String local_time;
  int traj_id;
  String season;
  int cluster_id;
  
  TrackPt(int pt_id, float lon, float lat, float speed, float heading, String name, 
  String local_time, int traj_id, String season, int cluster_id) {
    this.pt_id = pt_id;
    this.lon = lon;
    this.lat = lat;
    this.speed = speed;
    this.heading = heading;
    this.name = name;
    this.local_time = local_time;
    this.traj_id = traj_id;
    this.season = season;
    this.cluster_id = cluster_id;
  }
}


class Trajectory {
  int traj_id;
  String season;
  int cluster_id;
  ArrayList<TrackPt> tj_track_pts;
  
  Trajectory (ArrayList<TrackPt> tj_track_pts, int traj_id, String season, int cluster_id) {
    this.tj_track_pts = tj_track_pts;
    this.traj_id = traj_id;
    this.season = season;
    this.cluster_id = cluster_id;
  }
}


class SamplePt {
  int pt_id;
  float lon;
  float lat;
  String season;
  float thermal_uplift; 
  float temperature;
  float ndvi;
  float precipitation;
  float tailwind;
  float crosswind;
  
  SamplePt(int pt_id, float lon, float lat, String season, float thermal_uplift,
  float temperature, float ndvi, float precipitation, float tailwind, float crosswind) {
    this.pt_id = pt_id;
    this.lon = lon;
    this.lat = lat;
    this.season = season;
    this.thermal_uplift = thermal_uplift;
    this.temperature = temperature;
    this.ndvi = ndvi;
    this.precipitation = precipitation;
    this.tailwind = tailwind;
    this.crosswind = crosswind;
  }
}


/*
 * Smoke Particle System
 * Simple Particle System
 * by Daniel Shiffman.
 *
 * A basic smoke effect using a particle system. Each particle
 * is rendered as an alpha masked image.
*/

// A simple Particle class, renders the particle as an image
class Particle {
  PVector loc;
  PVector vel;
  PVector acc;
  float lifespan;
  PImage img;
  float tint_val;

  Particle(PVector l, PImage img_, float tint_val_) {
    acc = new PVector(0, 0);
    float vx = randomGaussian()*0.3;
    float vy = randomGaussian()*0.3 - 1.0;
    vel = new PVector(vx, vy);
    loc = l.copy();
    lifespan = 60.0;
    img = img_;
    tint_val = tint_val_;
  }

  void run() {
    update();
    render(tint_val);
  }

  // Method to apply a force vector to the Particle object
  // Note we are ignoring "mass" here
  void applyForce(PVector f) {
    acc.add(f);
  }  

  // Method to update position
  void update() {
    vel.add(acc);
    loc.add(vel);
    lifespan -= 2.5;
    acc.mult(0); // clear Acceleration
  }

  // Method to display
  void render(float tint_val) {
    imageMode(CENTER);
    tint(tint_val, lifespan);
    
    img.resize(0, 3);  // resize img to make it smaller
    
    image(img, loc.x, loc.y);
    // Drawing a circle instead
    // fill(255,lifespan);
    // noStroke();
    // ellipse(loc.x,loc.y,img.width,img.height);
  }

  // Is the particle still useful?
  boolean isDead() {
    if (lifespan <= 0.0) {
      return true;
    } else {
      return false;
    }
  }
}


// A class to describe a group of Particles
// An ArrayList is used to manage the list of Particles 
class ParticleSystem {

  ArrayList<Particle> particles;    // An arraylist for all the particles
  PVector origin;                   // An origin point for where particles are birthed
  PImage img;
  float tint_val;

  ParticleSystem(int num, PVector v, PImage img_, float tint_val_) {
    particles = new ArrayList<Particle>();              // Initialize the arraylist
    origin = v.copy();                                   // Store the origin point
    img = img_;
    tint_val = tint_val_;
    for (int i = 0; i < num; i++) {
      particles.add(new Particle(origin, img, tint_val));         // Add "num" amount of particles to the arraylist
    }
  }

  void run() {
    for (int i = particles.size()-1; i >= 0; i--) {
      Particle p = particles.get(i);
      p.run();
      if (p.isDead()) {
        particles.remove(i);
      }
    }
  }

  // Method to add a force vector to all particles currently in the system
  void applyForce(PVector dir) {
    // Enhanced loop!!!
    for (Particle p : particles) {
      p.applyForce(dir);
    }
  }  

  void addParticle() {
    particles.add(new Particle(origin, img, tint_val));
  }
}
