void loadData() {
  Table sample_table;
  Table track_table;
  
  sample_table = loadTable("Sample_Pts_wEnv_TailCross.csv", "header");
  track_table = loadTable("Clustered_Pts_Lat30N.csv", "header");
  
  n_sample_pts = sample_table.getRowCount();
  n_track_pts = track_table.getRowCount();
  println("Number of rows in the sample dataset:", n_sample_pts);
  println("Number of rows in the tracking dataset:", n_track_pts);
  
  // sample pts
  for (int i=0; i<n_sample_pts; i++) {
    int pt_id = sample_table.getInt(i, 0);
    
    float lon = sample_table.getFloat(i, 1);
    float lat = sample_table.getFloat(i, 2);
    
    // min, max lon, lat
    if (lon < lon_min) {
      lon_min = lon;
    } else if (lon > lon_max) {
      lon_max = lon;
    }
    if (lat < lat_min) {
      lat_min = lat;
    } else if (lat > lat_max) {
      lat_max = lat;
    }
    
    String season = sample_table.getString(i, 4);
    float thermal_uplift = sample_table.getFloat(i, 5);
    float temperature = sample_table.getFloat(i, 6);
    float ndvi = sample_table.getFloat(i, 7);
    float precipitation = sample_table.getFloat(i, 11);
    float tailwind = sample_table.getFloat(i, 13);
    float crosswind = sample_table.getFloat(i, 14);
    
    // min, max thermal_uplift, temperature, ndvi, precipitation, tailwind, crosswind
    if (thermal_uplift < thermal_min) {
      thermal_min = thermal_uplift;
    } else if (thermal_uplift > thermal_max) {
      thermal_max = thermal_uplift;
    }
    if (temperature < temp_min) {
      temp_min = temperature;
    } else if (temperature > temp_max) {
      temp_max = temperature;
    }
    if (ndvi < ndvi_min) {
      ndvi_min = ndvi;
    } else if (ndvi > ndvi_max) {
      ndvi_max = ndvi;
    }
    if (precipitation < precip_min) {
      precip_min = precipitation;
    } else if (precipitation > precip_max) {
      precip_max = precipitation;
    }
    if (tailwind < tail_min) {
      tail_min = tailwind;
    } else if (tailwind > tail_max) {
      tail_max = tailwind;
    }
    if (crosswind < cross_min) {
      cross_min = crosswind;
    } else if (crosswind > cross_max) {
      cross_max = crosswind;
    }
    
    sample_pts.add(new SamplePt(pt_id, lon, lat, season, thermal_uplift, temperature, 
    ndvi, precipitation, tailwind, crosswind));
    
    // add to terrain background_env
    float[] lonlat = {lon, lat};
    float[] xy = lonlat2xy(lonlat);
    int spi_col = int(xy[0] / scl);
    int spi_row = int(xy[1] / scl);
    background_env[spi_col][spi_row][0] = thermal_uplift;
    background_env[spi_col][spi_row][1] = temperature;
    background_env[spi_col][spi_row][2] = ndvi;
    background_env[spi_col][spi_row][3] = precipitation;
    background_env[spi_col][spi_row][4] = tailwind;
    background_env[spi_col][spi_row][5] = crosswind;
  }
  
  //println("Minimal temperature: ", temp_min, "Maximum temperature: ", temp_max);
  
  // tracking pts
  for (int i=0; i<n_track_pts; i++) {
    int pt_id = track_table.getInt(i, 0);
    float lon = track_table.getFloat(i, 1);
    float lat = track_table.getFloat(i, 2);
    float speed = track_table.getFloat(i, 3);
    float heading = track_table.getFloat(i, 4);
    String name = track_table.getString(i, 5);
    String local_time = track_table.getString(i, 6);
    int traj_id = track_table.getInt(i, 9);
    String season = track_table.getString(i, 10);
    int cluster_id = track_table.getInt(i, 11);
    
    track_pts.add(new TrackPt(pt_id, lon, lat, speed, heading, name, local_time,
    traj_id, season, cluster_id));
  }
  
  // println(sample_pts.get(0).thermal_uplift);
  // println(track_pts.get(0).local_time);
  
}


float[] lonlat2xy(float[] lonlat) {
  float x = map(lonlat[0], lon_min, lon_max, 0, w);
  float y = map(lonlat[1], lat_min, lat_max, 0, h);
  
  float[] xy = {x, y};
  
  return xy;
}


void form_trajs(ArrayList<TrackPt> track_pts, int n_track_pts) {
  ArrayList<TrackPt> tj_track_pts = new ArrayList<TrackPt>();
  int traj_id = -1;
  String season = "Fall";
  int cluster_id = 0;
  Trajectory traj;
  
  for (int i=0; i<n_track_pts; i++) {
    if (traj_id == -1) {
      // the first 
      traj_id++;
      tj_track_pts.add(track_pts.get(i));
      season = track_pts.get(i).season;
      cluster_id = track_pts.get(i).cluster_id;
      
    } else {
      // the following pts
      if (track_pts.get(i).traj_id == traj_id) {
      tj_track_pts.add(track_pts.get(i));
      } else {
        traj = new Trajectory(tj_track_pts, traj_id, season, cluster_id);
        trajectories.add(traj);
        
        traj_id ++;
        season = track_pts.get(i).season;
        cluster_id = track_pts.get(i).cluster_id;
        tj_track_pts = new ArrayList<TrackPt>();
        tj_track_pts.add(track_pts.get(i));
      }
    }
  }
  traj = new Trajectory(tj_track_pts, traj_id, season, cluster_id);
  trajectories.add(traj);
  
  n_trajs = trajectories.size();
  println("Number of trajectories:", trajectories.size());
}
