color env_terrain_color(float env_xyi, float fluctuation) {
  color c;
  // compute the scale to determine the color
  float env_scl = map(env_xyi, -fluctuation, fluctuation, 0, 1);
  if (env_scl > 0.8) {
    c = #E74C3C;
  } else if (env_scl > 0.6) {
    c = #F39C12;
  } else if (env_scl > 0.4) {
    c = #BDC3C7;
  } else if (env_scl > 0.2) {
    c = #1ABC9C;
  } else {
    c = #3498DB;
  }
  return c;
}


float norm_env(int x, int y, int i, float[][][] background_env) {
  float orig_env = background_env[x][y][i];
  float envi = 0;
  if (Float.isNaN(orig_env)) {
    envi = 0;
  } else {
      if (i == 0) {
      envi = map(background_env[x][y][i], 
              thermal_min, thermal_max,
              -fluctuation, fluctuation);
    } else if (i == 1) {
      envi = map(background_env[x][y][i], 
              temp_min, temp_max,
              -fluctuation, fluctuation);
    } else if (i == 2) {
      envi = map(background_env[x][y][i], 
              ndvi_min, ndvi_max,
              -fluctuation, fluctuation);
    } else if (i == 3) {
      envi = map(background_env[x][y][i], 
              precip_min, precip_max,
              -fluctuation, fluctuation);
    } else if (i == 4) {
      envi = map(background_env[x][y][i], 
              tail_min, tail_max,
              -fluctuation, fluctuation);
    } else if (i == 5) {
      envi = map(background_env[x][y][i], 
              cross_min, cross_max,
              -fluctuation, fluctuation);
    } 
  }
  return envi;
}


void visualize_sample_env_terrain(float[][][] background_env, float ver_dist) {
  pushMatrix();
  
  stroke(255);
  noFill();
  
  translate(width/2, height/2-300);
  rotateX(PI/3);
  translate(-w/2, -h/2);
  
  for (int i = 0; i < 6; i++) {
    // draw a terrain
    for (int y = 0; y < rows-1; y++) {
      beginShape(TRIANGLE_STRIP);
      for (int x = 0; x < cols; x++) {
        // background environmental variable
        float env_xyi = norm_env(x, y, i, background_env);
        if (env_xyi < -fluctuation) {
          env_xyi = -fluctuation;
        }
        color c = env_terrain_color(env_xyi, fluctuation);
        vertex(x*scl, y*scl, ter_base_z + i*ver_dist + env_xyi);
        if (color_terrain == true) {
          fill(c);
        }
        float env_xy1i = norm_env(x, y+1, i, background_env);
        if (env_xy1i < -fluctuation) {
          env_xy1i = -fluctuation;
        }
        color c1 = env_terrain_color(env_xy1i, fluctuation);
        vertex(x*scl, (y+1)*scl, ter_base_z + i*ver_dist + env_xy1i);
        if (color_terrain == true) {
          fill(c1);
        }
      }
      endShape();
    }
  }
  popMatrix();
}


void label_env_terrains(String[] envs, float ver_dist) {
  // environmental variable name corresponding to that terrain
  pushMatrix();
  
  translate(width/2, height/2-300);
  rotateX(PI/3);
  translate(-w/2, -h/2);
  
  textSize(400);
  float text_scl = 0.1;
  scale(text_scl);
  fill(210);
  
  for (int i = 0; i < 6; i++) {
  text(envs[i], (-w/2.5)/text_scl, (h/10)/text_scl, (ter_base_z + i*ver_dist)/text_scl);
  }
  
  popMatrix();
}


void draw_traj_pts(ArrayList<Trajectory> trajectories) {
  float x = 0.0;
  float y = 0.0;
  Trajectory traj;
  int traj_n_pts;
  ArrayList<TrackPt> tj_track_pts;
  
  float[] lonlat;
  float[] xy;
  
  for (int i=0; i<n_trajs; i++) {
    traj = trajectories.get(i);
    tj_track_pts = traj.tj_track_pts;
    traj_n_pts = tj_track_pts.size();
    //println("Number of pts in Traj", i, ":", traj_n_pts);
    
    for (int j=0; j<traj_n_pts; j++) {
      if (ms % traj_n_pts == j) {
        // lon, lat of the pt
      lonlat = new float[2];
      lonlat[0] = tj_track_pts.get(j).lon;
      lonlat[1] = tj_track_pts.get(j).lat;
      
      // map lon, lat to x, y
      xy = lonlat2xy(lonlat);
      x = xy[0];
      y = xy[1];
      
      pushMatrix();
      noStroke();
      fill(#FBFF00);
      
      translate(width/2, height/2-300);
      rotateX(PI/3);
      translate(-w/2, -h/2);
      
      translate(x, y, 1000);
      sphere(6);
      popMatrix();
      }
    }
  }
}


void draw_traj_lines(ArrayList<Trajectory> trajectories) {
  Trajectory traj;
  int traj_n_pts;
  ArrayList<TrackPt> tj_track_pts;
  
  float[] lonlat1, lonlat2;
  float[] xy1, xy2;
  float x1, y1, x2, y2;
  
  
  for (int i=0; i<n_trajs; i++) {
    traj = trajectories.get(i);
    tj_track_pts = traj.tj_track_pts;
    traj_n_pts = tj_track_pts.size();
    
    for (int j=0; j<traj_n_pts-1; j++) {
      // calculate coordinates
      lonlat1 = new float[2];
      lonlat1[0] = tj_track_pts.get(j).lon;
      lonlat1[1] = tj_track_pts.get(j).lat;
      
      lonlat2 = new float[2];
      lonlat2[0] = tj_track_pts.get(j+1).lon;
      lonlat2[1] = tj_track_pts.get(j+1).lat;
      
      xy1 = lonlat2xy(lonlat1);
      xy2 = lonlat2xy(lonlat2);
      
      x1 = xy1[0];
      y1 = xy1[1];
      x2 = xy2[0];
      y2 = xy2[1];
      
      // draw line  
      pushMatrix();
      translate(width/2, height/2-300);
      rotateX(PI/3);
      translate(-w/2, -h/2);
      translate(0, 0, 1000);
      stroke(#A70A0A);
      strokeWeight(2);
      line(x1, y1, x2, y2);
      popMatrix();
    }
  }
}
