/*************************************************************************************
 Student-defined visualization project 
 - How turkey vultures migrate in their surrounding environment  
 
 Tested in Processing 4                                    
 
 Author: Zijian Wan                       
 Supervisor: George Legrady                           
 
 Purpose: This project visualizes how animals interact with the surrounding environment 
 (i.e., how environmental variables might influence the movement path selection) using 
 the turkey vulture migration example.
 
 *************************************************************************************/
 
import peasy.*;
import controlP5.*;


int cols, rows;
int scl = 20;
int w = 1000;
int h = 2000;
float ver_dist=100.0;
float fluctuation = 40.0;
float ter_base_z = 0;  // the baseline z coordinate of the terrain
boolean color_terrain = false;
boolean vis_traj_lines = false;
boolean vis_traj_pts = false;

int n_sample_pts;
int n_track_pts;
int n_trajs;

int ms;

// initialize the value for min, max lon, lat
float lon_min = 1000;
float lon_max = -1000;
float lat_min = 1000;
float lat_max = -1000;

// initialize the value for min, max thermal_uplift, temperature, ndvi, precipitation, tailwind, crosswind
float thermal_min = 10000;
float thermal_max = -10000;
float temp_min = 10000;
float temp_max = -10000;
float ndvi_min = 10000;
float ndvi_max = -10000;
float precip_min = 10000;
float precip_max = -10000;
float tail_min = 10000;
float tail_max = -10000;
float cross_min = 10000;
float cross_max = -10000;
String[] envs = {"Thermal uplift", "Temperature", "NDVI", "Precipitation", "Tailwind",
                "Crosswind"};

// terrains for each environmental variable
float[][][] background_env;

PeasyCam cam;
ControlP5 cp5;
ParticleSystem ps;

ArrayList<TrackPt> track_pts = new ArrayList<TrackPt>();
ArrayList<SamplePt> sample_pts = new ArrayList<SamplePt>();
ArrayList<Trajectory> trajectories = new ArrayList<Trajectory>();

 
void setup() {
  // canvas
  size(3500, 2200, P3D);
  //fullScreen(P3D);
  
  // peasy cam
  cam = new PeasyCam(this, 0, 0, 0, 2000);
  cam.setMaximumDistance(4000);
  //cam.lookAt(1500, 150, 50);
  cam.lookAt(width/2, 150, 50);
  
  // control
  cp5 = new ControlP5(this);
  cp5.setAutoDraw(false);
  cp5.addToggle("color_terrain")
    .setPosition(width/10, height*0.3)
    .setSize(width/20,width/60);
  cp5.addToggle("vis_traj_lines")
    .setPosition(width/10, height*0.35)
    .setSize(width/20,width/60);
  cp5.addToggle("vis_traj_pts")
    .setPosition(width/10, height*0.4)
    .setSize(width/20,width/60);
  cp5.addSlider("fluctuation")
    .setRange(0, ver_dist)
    .setPosition(width/13, height*0.6)
    .setSize(width/10,width/100);
  cp5.addSlider("ver_dist")
    .setRange(10, 3000)
    .setPosition(width/13, height*0.65)
    .setSize(width/10,width/100);
  cp5.addSlider("ter_base_z")
    .setRange(-5000, 10000)
    .setPosition(width/13, height*0.7)
    .setSize(width/10,width/100);
  
  
  cols = w / scl + 1;
  rows = h / scl + 1;
  background_env = new float[cols][rows][6];
  
  // load data
  loadData();
  form_trajs(track_pts, n_track_pts);
  //frameRate(30);
  
  //gen_sample_pt_env_ps();
  
}
  

void draw() {
  background(0);
  
  // visualize the terrains of environmental variables
  visualize_sample_env_terrain(background_env, ver_dist);
  label_env_terrains(envs, ver_dist);
  
  //visualize_sample_env_smoke();
  
  // visualize the trajectories
  if (vis_traj_pts) {
    ms = int(millis() / 50);
    draw_traj_pts(trajectories);
  }
  if (vis_traj_lines) {
    draw_traj_lines(trajectories);
  }
  
  
  gui();
}


void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}
 
